﻿--[[$Id: GetInLine.lua 30626 2007-03-21 02:58:07Z eternally777 $]]

--[[
Outline of Goals:
   - FuBar Plug-in
   - OnMouseOver:
      * Display list of all players in line by
	* Class
	* Order Added
      * Display time spent on list for each person in parenthesis by their name
--]]
---------------------------------
--       Local Variables       --
---------------------------------
local _G = getfenv(0)
local L = AceLibrary('AceLocale-2.2'):new('GetInLine')
local T = AceLibrary('Tablet-2.0')


local playerName
local prevInRaid
local lastAuthor
local notificationFrames

-- This variable can hold the string formed by table.concat'ing the wait list.
-- Make sure you set it equal to 'nil' whenever the wait list changes!
local concatCache


local function FindValue(t, val)
	if not t then return end
	for i, v in ipairs(t) do
		if v == val then
			return i
		end
	end
end


local function DelTable(t)
	if not t then return end
	for i in ipairs(t) do
		if type(t[i]) == 'table' then
			DelTable(t[i])
		else
			t[i] = nil
		end
	end
end


local profileDefaults = {
	raidOnlyWhispers = true,
	clearOnExit = false,
	notifications = {
		additions = true,
		removals = true,
		listRequest = true,
		listSent = true
	}
}


local charDefaults = {}


---------------------------------
--      Addon Declaration      --
---------------------------------
GetInLine = AceLibrary('AceAddon-2.0'):new('AceEvent-2.0', 'AceHook-2.1', 'AceDB-2.0', 'AceConsole-2.0', 'AceComm-2.0', 'FuBarPlugin-2.0')
GetInLine.revision = ("$Revision: 30626 $"):sub(-7, -3)
GetInLine.version = GetInLine.version .. " |cffff8888r" .. GetInLine.revision .. "|r"
GetInLine.date = ("$Date: 2007-03-20 22:58:07 -0400 (Tue, 20 Mar 2007) $"):sub(8, 17)


---------------------------------
--        Initialization       --
---------------------------------
function GetInLine:OnInitialize()
	self:RegisterDB('GetInLineDB', 'GetInLinePCDB')
	self:RegisterDefaults('profile', profileDefaults)
	self:RegisterDefaults('char', charDefaults)
	self.commPrefix = 'GiL'
end


function GetInLine:OnEnable()
	playerName = UnitName('player')
	msgPrefix = 'GiL: %s'
	self:InRaid()
	
	self:RegisterEvent('CHAT_MSG_WHISPER')
	self:RegisterEvent('RAID_ROSTER_UPDATE', 'InRaid')
	self:RegisterComm(self.commPrefix, 'GUILD', 'OnCommReceive')
	
	for i = 1, 7 do
		self:Hook(_G['ChatFrame' .. i], 'AddMessage', true)
	end
	
	self:UpdateDisplay()
end


function GetInLine:OnDisable()
	playerName = nil
	msgPrefix = nil
	prevInRaid = nil
	lastAuthor = nil
	concatCache = nil
	self.db.char.waitList = DelTable(self.db.char.waitList)
end


---------------------------------
--        Remote Commands      --
---------------------------------
local remoteCommands = {
	[ L['Add']:lower() ] = function(self, author)
		if self:AddPlayer(author) then
			self:Respond(L['You have added %s to the wait list.']:format(L['yourself']))
			if self.db.profile.notifications.additions then
				self:Print(L['%s has added %s to the wait list.']:format(author, L['his or her self']))
			end
		else
			self:Respond(L['You are already on the list. To view the list, please respond with "!gil list".'])
		end
	end,
	[ L['Rem']:lower() ] = function(self, author)
		if self:RemovePlayer(author) then
			self:Respond(L['You have removed %s from the wait list.']:format(L['yourself']))
			if self.db.profile.notifications.removals then
				self:Print(L['%s has removed %s from the wait list.']:format(author, L['his or her self']))
			end
		else
			self:Respond(L['You are not currently on the list. To view the list, please respond with "!gil list".'])
		end
	end,
	[ L['List']:lower() ] = function(self, author)
		if not self.db.char.waitList then
			self:Respond(L['The wait list is currently empty.'])
		else
			if not concatCache then
				concatCache = table.concat(self.db.char.waitList, ', ')
			end
			self:Respond(L['The List - '] .. concatCache .. '.')
		end
	end
}


---------------------------------
--        Event Handlers       --
---------------------------------
function GetInLine:CHAT_MSG_WHISPER(message, author, ...)
	message = message:lower()
	if not message:find('^!gil') then
		return
	end
	
	lastAuthor = author
	
	local command = select(3, message:find('^!gil%s(%w+)'))
	
	if not command then
		self:Respond(L['To add yourself to the wait list, respond with "!gil add".  To remove yourself, respond with "!gil rem".'])
		return
	end
	
	for k in next, remoteCommands do
		if k == command then
			remoteCommands[k](self, author)
			break
		end
	end
	
	-- Nilling this variable here makes sense because
	--  this function is called once after each whisper.
	--  Once we've responded to the whisper if necessary,
	--  then we no longer need to know who the author was.
	lastAuthor = nil
end


---------------------------------
--         Comm Handler        --
---------------------------------
local metaComm = { __index = function() error('Nonexistent function called') end }
local commHandler = {
	enable = function(sender)
		
	end,
}
setmetatable(commHandler, metaComm)
function GetInLine:OnCommReceive(prefix, sender, distro, message, ...)
	commHandler[message](self, sender, ...)
end


---------------------------------
--        Hook Handlers        --
---------------------------------
function GetInLine:AddMessage(frame, text, ...)
	-- This will hide all whisper text. Our event handler will manage notifications.
	if not text:find('GiL:%s') and not text:find('!gil%s') then
		return self.hooks[frame].AddMessage(frame, text, ...)
	end
end


---------------------------------
--      FuBar 2.0 Methods      --
---------------------------------
function GetInLine:OnTextUpdate()
	local numOnList = self.db.char.waitList and #self.db.char.waitList or 0
	self:SetText(L['%d Waiting']:format(numOnList))
end


function GetInLine:OnTooltipUpdate()
	local cat
	if not self.db.char.waitList or #self.db.char.waitList == 0 then 
		
		cat = T:AddCategory(
			'child_textR', 0,
			'child_textG', 1,
			'child_textB', 0
		)
		
		cat:AddLine(
			'text', L['The wait list is currently empty.']
		)
		
	else
		
		cat = T:AddCategory(
			'child_textR', 1,
			'child_textG', 1,
			'child_textB', 1
		)
		
		for i, v in ipairs(self.db.char.waitList) do
			cat:AddLine(
				'text', v
			)
		end
		
	end
end


---------------------------------
--        Other Methods        --
---------------------------------
function GetInLine:RemovePlayer(playerName)
	-- if not self:InRaid() then return end
	local index = FindValue(self.db.char.waitList, playerName)
	
	if index then
		tremove(self.db.char.waitList, index)
		
		if #self.db.char.waitList == 0 then
			self.db.char.waitList = nil
		end
		
		concatCache = nil
		self:UpdateDisplay()
	end
	
	return index
end


function GetInLine:AddPlayer(playerName)
	-- if not self:InRaid() then return end
	if self.db.char.waitList and FindValue(self.db.char.waitList, playerName) then
		return
	end
	
	if not self.db.char.waitList then
		self.db.char.waitList = { playerName }
	else
		tinsert(self.db.char.waitList, playerName)
	end
	
	concatCache = nil
	self:UpdateDisplay()
	
	return true
end


function GetInLine:InRaid()
	local inRaid = GetNumRaidMembers() > 0 or nil
	
	if inRaid and not prevInRaid and not self:IsEventRegistered('CHAT_MSG_WHISPER') then
		self:RegisterEvent('CHAT_MSG_WHISPER')
	elseif prevInRaid and not inRaid and self.db.profile.clearOnExit then
--		self:UnregisterEvent('CHAT_MSG_WHISPER')
		self.db.char.waitList = DelTable(self.db.char.waitList)
		concatCache = nil
	end
	
	prevInRaid = inRaid
end


function GetInLine:Respond(message)
	if not message then return end
	if not lastAuthor then
		error('No last author detected.')
	end
	
	SendChatMessage(msgPrefix:format(message), 'WHISPER', nil, lastAuthor)
	lastAuthor = nil
end


function GetInLine:GetListSize()
	if self.db.char.waitList then
		return #self.db.char.waitList
	end
end


function GetInLine:AnnounceList(channel)
	if not self.db.char.waitList then
		self:Print(L['The wait list is currently empty.'])
	else
		if not concatCache then
			concatCache = table.concat(self.db.char.waitList, ', ')
		end
		SendChatMessage(msgPrefix:format(L['The List - '] .. concatCache .. '.'), channel)
	end
end


function GetInLine:PurgeList()
	local self = GetInLine
	if not self.db.char.waitList then return end
	self.db.char.waitList = DelTable(self.db.char.waitList)
	self:Print('The wait list has been purged.')
end
